/*
    Class CObjReference: The Implementation
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "CObjReference.h"

// Default constructor
CObjReference::CObjReference()
{
    NumOfObject = 0;
    ObjectList = NULL;
}

// Construct form files on path
// Object must be in object.library and material must be in material.library
// Input parameter: path = object & material path
CObjReference::CObjReference(const char *path)
{
    NumOfObject = 0;
    ObjectList = NULL;
    OpenLibrary(path);
}

// Destructor... Cleaning all object available from the memory and VRAM
CObjReference::~CObjReference()
{
    // Remove from display list
    for (int i = 0; (unsigned int) i < NumOfObject; i++)
        glDeleteLists(ObjectList[i].DisplayListNum, 1);
    // Free memory
    if(ObjectList)
        delete ObjectList;
    #ifdef _DEBUG_
        fprintf(stderr, "Cleaning reference objects\n");
    #endif
}

// Open library located in path
// Return value: number of objects if success, 0 if unsuccess
unsigned int CObjReference::OpenLibrary(const char *path)
{
    FILE *nus_file = NULL;
    char objectpath[256];
    char mtlpath[256];
    char librarylist[256];
    char objectname[256];
    char ReadableID[256];
    char mtlname[256];
    char objectfullpath[256];
    char mtlfullpath[256];
    char buffer[64];
    int i;

    // Remove from display list
    for (i = 0; (unsigned int) i < NumOfObject; i++)
        glDeleteLists(ObjectList[i].DisplayListNum, 1);
    // Free memory
    if(ObjectList)
        delete ObjectList;
    
    // Generate path to objects & materials
    sprintf(objectpath, "%s/object.library", path);
    sprintf(mtlpath, "%s/material.library", path);
    sprintf(librarylist, "%s/library.list", path);
    #ifdef _DEBUG_
        fprintf(stderr, "Library list: %s\n", librarylist);
        fprintf(stderr, "Object path: %s\n", objectpath);
        fprintf(stderr, "Material path: %s\n", mtlpath);
    #endif
    
    // First initialization
    NumOfObject = 0;
    ObjectList = NULL;
    
    // Read the list containing list of object along with their materials
    if((nus_file = fopen(librarylist, "r")) != NULL)
    {
        // Getting number of objects in the list
        fscanf(nus_file, "%[^:]:", buffer);
        #ifdef _DEBUG_
            fprintf(stderr, "%s = ", buffer);
        #endif
        fscanf(nus_file, "%[^\n]\n", buffer);
        NumOfObject = (unsigned int) atoi(buffer);
        #ifdef _DEBUG_
            fprintf(stderr, "%d\n", NumOfObject);
        #endif
        // Allocating memory for the list
        ObjectList = new Object3D[NumOfObject];
        if(ObjectList)
        {
            // Reading the list
            for(i = 0; (unsigned int) i < NumOfObject && !feof(nus_file); i++)
            {
                fscanf(nus_file, "%s %s %s\n", ReadableID, objectname, mtlname);
                sprintf(objectfullpath, "%s/%s", objectpath, objectname);
                sprintf(mtlfullpath, "%s/%s", mtlpath, mtlname);
                #ifdef _DEBUG_
                    fprintf(stderr, "Reading object %s...\n", objectname);
                    fprintf(stderr, "Reading material %s...\n", mtlname);
                #endif
                // Reads and creates a display list all the object
                ReadMaterialLibrary(mtlfullpath);
                ReadObject(objectfullpath, &ObjectList[i]);
                strncpy(ObjectList[i].Name, ReadableID, 256);
                ObjectList[i].Name[255] = 0x00;
            }
            // The number of objects in the file is less than declared
            if((unsigned int) i < NumOfObject)
                NumOfObject = i;
            return NumOfObject;
        }
    }
    return(0);    
}

// Refer to an object in the list
// Input parameter: index = list index
// Output parameter: object = refered object
// Return value: index if success, -1 if unsuccess
int CObjReference::ReferObjectAt(unsigned int index, Object3D *object)
{
    if(index >= NumOfObject)
    {
        #ifdef _DEBUG_
            fprintf(stderr, "Cannot allocate memory...\n");
        #endif
        return(-1);
    }
    if(!object)
        if((object = new Object3D) == NULL)
        {
            #ifdef _DEBUG_
                fprintf(stderr, "Cannot allocate memory...\n");
            #endif
            return(-1);
        }
    strncpy(object->Name, ObjectList[index].Name, 256);
    object->Name[255] = 0x00;
    object->DisplayListNum = ObjectList[index].DisplayListNum;
    object->MinimumBoundingBox.x = ObjectList[index].MinimumBoundingBox.x;
    object->MinimumBoundingBox.y = ObjectList[index].MinimumBoundingBox.y;
    object->MinimumBoundingBox.z = ObjectList[index].MinimumBoundingBox.z;
    object->MaximumBoundingBox.x = ObjectList[index].MaximumBoundingBox.x;
    object->MaximumBoundingBox.y = ObjectList[index].MaximumBoundingBox.y;
    object->MaximumBoundingBox.z = ObjectList[index].MaximumBoundingBox.z;
    return index;
}

// Return the number of objects in the table
// Return value: number of objects in the list
unsigned int CObjReference::NumberOfObjects(void)
{
    return(NumOfObject);
}
