/*
    Class: CScenegraph
    Created by Nus
    Description: class for my scene graph.
*/

#ifndef __CLASS_SCENEGRAPH__
#define __CLASS_SCENEGRAPH__

#include <string.h>
#include "objfile.h"

/*
    Scene graph nodes, implemented as a linked list.
*/

typedef struct TMaterial
{
    GLfloat Red;
    GLfloat Green;
    GLfloat Blue;
    GLfloat Alpha;
    void operator = (struct TMaterial in)
    {
        Red = in.Red;
        Green = in.Green;
        Blue = in.Blue;
        Alpha = in.Blue;
    };
} MaterialProperties;

typedef struct TSceneGraph
{
    GLuint ObjID;
    unsigned int Locked;
    Point3D ObjLocation;
    Point3D ObjRotation;
    Point3D ObjScaling;
    MaterialProperties ObjAmbient;
    MaterialProperties ObjDiffuse;
    MaterialProperties ObjSpecular;
    GLuint ObjShininess;
    unsigned int ReferenceNum;
    char ObjInfo[1024];
    struct TSceneGraph *NextNode;
    void operator = (struct TSceneGraph input)
    {
        ObjID = input.ObjID;
        ObjLocation.x = input.ObjLocation.x;
        ObjLocation.y = input.ObjLocation.y;
        ObjLocation.z = input.ObjLocation.z;
        ObjRotation.x = input.ObjRotation.x;
        ObjRotation.y = input.ObjRotation.y;
        ObjRotation.z = input.ObjRotation.z;
        ObjScaling.x = input.ObjScaling.x;
        ObjScaling.y = input.ObjScaling.y;
        ObjScaling.z = input.ObjScaling.z;
        ObjAmbient = input.ObjAmbient;
        ObjDiffuse = input.ObjDiffuse;
        ObjSpecular = input.ObjSpecular;
        ObjShininess = input.ObjShininess;
        ReferenceNum = input.ReferenceNum;
        strncpy(ObjInfo, input.ObjInfo, 1024);
        ObjInfo[1023] = 0x00;
    };
} SceneGraph;

/*
    CSceneGraph: The Definition
*/
class CSceneGraph
{
private:
    SceneGraph *LastNode;
    SceneGraph *NodePtr;
protected:
    unsigned int NumOfNodes;                // Number of nodes in graph
    SceneGraph *Nodes;                      // The nodes themself
    bool SceneChanged;			    // Graph modified flag
    GLuint CounterID;			    // New object ID counter...
public:
    CSceneGraph();                          // Constructor
    ~CSceneGraph();                         // Destructor
    int AddNode(SceneGraph node, bool cnt); // Add node as the last node
    int DeleteNode(unsigned int index);     // Delete node at n
    int GetNodeAt(unsigned int index, SceneGraph *node);    // Get node at n
    int DeleteNodeWithID(GLuint theID);	    // Delete object based on objID
    int GetNodeWithID(GLuint theID, SceneGraph *node);
                                            // Get object based on objID
    void RewindNodePointer(void);           // Rewind pointer to the beginning
    int GetCurrentNodePointer(SceneGraph *node);            // Get current node
    int UpdateNode(SceneGraph node);	    // Update a node
    int Serialize(const char *filename);    // Serialize scene graph
    int LoadFromFile(const char *filename); // Load scene graph
    void DeleteAllNodes(void);              // Deleting all existing nodes
    unsigned int NumberOfNodes(void);       // Return number of nodes available
};

#endif
